@@ -5,6 +5,7 @@
 # That should not affect neither the public API, nor the
 # type matching with isinstance() and issubclass()
 #
+import pkg_resources
 import sys
 import struct
 import logging
@@ -18,9 +19,11 @@ from pyroute2.netlink.rtnl.req import (IPRouteRequest,
                                        IPLinkRequest)
 from pyroute2.iproute import (IPRoute,
                               IPBatch,
-                              RawIPRoute)
+                              RawIPRoute,
+                              RemoteIPRoute)
 from pyroute2.ipset import IPSet
 from pyroute2.ipdb.main import IPDB
+from pyroute2.ndb.main import NDB
 from pyroute2.iwutil import IW
 from pyroute2.devlink import DL
 from pyroute2.conntrack import Conntrack
@@ -55,7 +58,7 @@ log = logging.getLogger(__name__)
 # on logging calls when no handler is configured.
 # see https://docs.python.org/2/howto/logging.html#library-config
 if sys.version_info >= (2, 7):  # This is only available from 2.7 onwards
-    log.addHandler(logging.NullHandler())
+    logging.getLogger('').addHandler(logging.NullHandler())
 
 try:
     # probe, if the bytearray can be used in struct.unpack_from()
@@ -86,7 +89,9 @@ classes = [IPRouteRequest,
            IPRoute,
            IPBatch,
            RawIPRoute,
+           RemoteIPRoute,
            IPSet,
+           NDB,
            IPDB,
            IW,
            DL,
@@ -125,3 +130,8 @@ globals()['ipdb'].common = __common()
 
 __all__.extend([x.__name__ for x in exceptions])
 __all__.extend([x.__name__ for x in classes])
+
+try:
+    __version__ = pkg_resources.get_distribution('pyroute2').version
+except:
+    __version__ = 'unknown'
