@@ -58,6 +58,7 @@ from pyroute2.netlink.rtnl import ndmsg
 from pyroute2.netlink.rtnl.ndtmsg import ndtmsg
 from pyroute2.netlink.rtnl.fibmsg import fibmsg
 from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
+from pyroute2.netlink.rtnl.ifinfmsg import IFF_NOARP
 from pyroute2.netlink.rtnl.ifaddrmsg import ifaddrmsg
 from pyroute2.netlink.rtnl.iprsocket import IPRSocket
 from pyroute2.netlink.rtnl.iprsocket import IPBatchSocket
@@ -503,7 +504,7 @@ class RTNL_API(object):
                     +------+                    +-----+
                                                    ^
                     +------+      +----------+     |
-            net --> | eth2 | <--> | eth0.500 | <---+
+            net --> | eth2 | <--> | eth2.500 | <---+
                     +------+      +----------+
 
         It means that one has to create as many bridges, as there were
@@ -887,6 +888,21 @@ class RTNL_API(object):
         Support for GRE over IPv6 is also included; use `kind=ip6gre` and
         `ip6gre_` as the prefix for its values.
 
+        ► ipip
+
+        Create ipip tunnel::
+
+            ip.link("add",
+                    ifname="tun1",
+                    kind="ipip",
+                    ipip_local="172.16.0.1",
+                    ipip_remote="172.16.0.101",
+                    ipip_ttl=16)
+
+        Support for sit and ip6tnl is also included; use `kind=sit` and `sit_`
+        as prefix for sit tunnels, and `kind=ip6tnl` and `ip6tnl_` prefix for
+        ip6tnl tunnels.
+
         ► macvlan
 
         Macvlan interfaces act like VLANs within OS. The macvlan driver
@@ -976,9 +992,10 @@ class RTNL_API(object):
 
             # create internal vlan 802.1q, c-tag
             ip.link("add",
-                    ifname="v100c",
+                    ifname="v200c",
                     kind="vlan",
                     link=ip.link_lookup(ifname="v100s")[0],
+                    vlan_id=200,
                     vlan_protocol=0x8100)
 
 
@@ -1028,6 +1045,35 @@ class RTNL_API(object):
             state="up":   flags=1, mask=1
             state="down": flags=0, mask=0
 
+        SR-IOV virtual function setup::
+
+            # get PF index
+            x = ip.link_lookup(ifname="eth0")[0]
+            # setup macaddr
+            ip.link("set",
+                    index=x,                          # PF index
+                    vf={"vf": 0,                      # VF index
+                        "mac": "00:11:22:33:44:55"})  # address
+            # setup vlan
+            ip.link("set",
+                    index=x,           # PF index
+                    vf={"vf": 0,       # VF index
+                        "vlan": 100})  # the simplest case
+            # setup QinQ
+            ip.link("set",
+                    index=x,                           # PF index
+                    vf={"vf": 0,                       # VF index
+                        "vlan": [{"vlan": 100,         # vlan id
+                                  "proto": 0x88a8},    # 802.1ad
+                                 {"vlan": 200,         # vlan id
+                                  "proto": 0x8100}]})  # 802.1q
+
+        **update**
+
+        Almost the same as `set`, except it uses different flags
+        and message type. Mostly does the same, but in some cases
+        differs. If you're not sure what to use, use `set`.
+
         **del**
 
         Destroy the interface::
@@ -1044,11 +1090,9 @@ class RTNL_API(object):
 
             ip.link("get", index=ip.link_lookup(ifname="br0")[0])
 
-        **vlan-add**
-        **vlan-del**
+        Get extended attributes like SR-IOV setup::
 
-        These command names are confusing and thus are deprecated.
-        Use `IPRoute.vlan_filter()`.
+            ip.link("get", index=3, ext_mask=1)
         '''
         if (command == 'dump') and ('match' not in kwarg):
             match = kwarg
@@ -1063,7 +1107,8 @@ class RTNL_API(object):
         flags_dump = NLM_F_REQUEST | NLM_F_DUMP
         flags_req = NLM_F_REQUEST | NLM_F_ACK
         flags_create = flags_req | NLM_F_CREATE | NLM_F_EXCL
-        commands = {'set': (RTM_SETLINK, flags_create),
+        commands = {'set': (RTM_NEWLINK, flags_req),
+                    'update': (RTM_SETLINK, flags_create),
                     'add': (RTM_NEWLINK, flags_create),
                     'del': (RTM_DELLINK, flags_create),
                     'remove': (RTM_DELLINK, flags_create),
@@ -1097,6 +1142,12 @@ class RTNL_API(object):
                 flags = 1             # 0 (down) or 1 (up)
             del kwarg['state']
 
+        # arp on/off shortcut
+        if 'arp' in kwarg:
+            mask |= IFF_NOARP
+            if not kwarg.pop('arp'):
+                flags |= IFF_NOARP
+
         msg['flags'] = flags
         msg['change'] = mask
 
@@ -1131,7 +1182,7 @@ class RTNL_API(object):
         * mask -- address mask
         * family -- socket.AF_INET for IPv4 or socket.AF_INET6 for IPv6
         * scope -- the address scope, see /etc/iproute2/rt_scopes
-        * \*\*kwarg -- any ifaddrmsg field or NLA
+        * kwarg -- dictionary, any ifaddrmsg field or NLA
 
         Later the method signature will be changed to::
 
@@ -1218,7 +1269,7 @@ class RTNL_API(object):
         # work on NLA
         for key in kwarg:
             nla = ifaddrmsg.name2nla(key)
-            if kwarg[key] is not None:
+            if kwarg[key] not in (None, ''):
                 msg['attrs'].append([nla, kwarg[key]])
 
         ret = self.nlm_request(msg,
