@@ -316,13 +316,14 @@ class NetlinkMixin(object):
         self.callbacks = []     # [(predicate, callback, args), ...]
         self.pthread = None
         self.closed = False
+        self.uname = config.uname
         self.capabilities = {'create_bridge': config.kernel > [3, 2, 0],
                              'create_bond': config.kernel > [3, 2, 0],
                              'create_dummy': True,
                              'provide_master': config.kernel[0] > 2}
         self.backlog_lock = threading.Lock()
         self.read_lock = threading.Lock()
-        self.sys_lock = threading.Lock()
+        self.sys_lock = threading.RLock()
         self.change_master = threading.Event()
         self.lock = LockFactory()
         self._sock = None
@@ -362,6 +363,9 @@ class NetlinkMixin(object):
         # Set defaults
         self.post_init()
 
+    def post_init(self):
+        pass
+
     def clone(self):
         return type(self)(family=self.family)
 
@@ -519,8 +523,8 @@ class NetlinkMixin(object):
     def recv_into(self, *argv, **kwarg):
         return self._recv_into(*argv, **kwarg)
 
-    def recv_ft(self, size=None):
-        return self._recv(size)
+    def recv_ft(self, *argv, **kwarg):
+        return self._recv(*argv, **kwarg)
 
     def async_recv(self):
         poll = select.poll()
@@ -825,29 +829,40 @@ class NetlinkMixin(object):
 
         msg_seq = self.addr_pool.alloc()
         with self.lock[msg_seq]:
-            try:
-                self.put(msg, msg_type, msg_flags, msg_seq=msg_seq)
-                for msg in self.get(msg_seq=msg_seq,
-                                    terminate=terminate,
-                                    callback=callback):
-                    yield msg
-
-            except Exception:
-                raise
-            finally:
-                # Ban this msg_seq for 0xff rounds
-                #
-                # It's a long story. Modern kernels for RTM_SET.*
-                # operations always return NLMSG_ERROR(0) == success,
-                # even not setting NLM_F_MULTY flag on other response
-                # messages and thus w/o any NLMSG_DONE. So, how to detect
-                # the response end? One can not rely on NLMSG_ERROR on
-                # old kernels, but we have to support them too. Ty, we
-                # just ban msg_seq for several rounds, and NLMSG_ERROR,
-                # being received, will become orphaned and just dropped.
-                #
-                # Hack, but true.
-                self.addr_pool.free(msg_seq, ban=0xff)
+            retry_count = 0
+            while True:
+                try:
+                    self.put(msg, msg_type, msg_flags, msg_seq=msg_seq)
+                    for msg in self.get(msg_seq=msg_seq,
+                                        terminate=terminate,
+                                        callback=callback):
+                        yield msg
+                    break
+                except NetlinkError as e:
+                    if e.code != 16:
+                        raise
+                    if retry_count >= 30:
+                        raise
+                    print('Error 16, retry {}.'.format(retry_count))
+                    time.sleep(0.3)
+                    retry_count += 1
+                    continue
+                except Exception:
+                    raise
+                finally:
+                    # Ban this msg_seq for 0xff rounds
+                    #
+                    # It's a long story. Modern kernels for RTM_SET.*
+                    # operations always return NLMSG_ERROR(0) == success,
+                    # even not setting NLM_F_MULTY flag on other response
+                    # messages and thus w/o any NLMSG_DONE. So, how to detect
+                    # the response end? One can not rely on NLMSG_ERROR on
+                    # old kernels, but we have to support them too. Ty, we
+                    # just ban msg_seq for several rounds, and NLMSG_ERROR,
+                    # being received, will become orphaned and just dropped.
+                    #
+                    # Hack, but true.
+                    self.addr_pool.free(msg_seq, ban=0xff)
 
 
 class BatchAddrPool(object):
