@@ -2,6 +2,7 @@ import os
 import sys
 import struct
 import pkgutil
+import logging
 import importlib
 from socket import AF_INET
 from socket import AF_INET6
@@ -13,10 +14,18 @@ from pyroute2.netlink import nla
 from pyroute2.netlink import nlmsg
 from pyroute2.netlink import nlmsg_atoms
 from pyroute2.netlink.rtnl.iw_event import iw_event
-from pyroute2.netlink.rtnl.ifinfmsg import plugins
-
-config.data_plugins_pkgs.append(plugins)
+from pyroute2.netlink.rtnl.ifinfmsg.plugins import (bond,
+                                                    gtp,
+                                                    ipvlan,
+                                                    tuntap,
+                                                    vlan,
+                                                    vrf,
+                                                    vti,
+                                                    vti6,
+                                                    vxlan,
+                                                    xfrm)
 
+log = logging.getLogger(__name__)
 
 # it's simpler to double constants here, than to change all the
 # module layout; but it is a subject of the future refactoring
@@ -85,6 +94,20 @@ IFF_VOLATILE = IFF_LOOPBACK |\
     IFF_LOWER_UP |\
     IFF_DORMANT
 
+##
+#
+# gre flags
+#
+GRE_ACK = 0x0080
+GRE_REC = 0x0700
+GRE_STRICT = 0x0800
+GRE_SEQ = 0x1000
+GRE_KEY = 0x2000
+GRE_ROUTING = 0x4000
+GRE_CSUM = 0x8000
+
+(GRE_NAMES, GRE_VALUES) = map_namespace('GRE_', globals())
+
 ##
 #
 # vlan filter flags
@@ -187,6 +210,19 @@ def load_plugins_by_pkg(pkg):
 
 data_plugins = {}
 
+for module in (bond,
+               gtp,
+               ipvlan,
+               tuntap,
+               vlan,
+               vrf,
+               vti,
+               vti6,
+               vxlan,
+               xfrm):
+    name = module.__name__.split('.')[-1]
+    data_plugins[name] = getattr(module, name)
+
 for pkg in config.data_plugins_pkgs:
     data_plugins.update(load_plugins_by_pkg(pkg))
 
@@ -284,6 +320,30 @@ class macvx_data(nla):
         nla_map = ((4, 'IFLA_MACVLAN_MACADDR', 'l2addr'), )
 
 
+class iptnl_data(nla):
+    nla_map = (('IFLA_IPIP_UNSPEC', 'none'),
+               ('IFLA_IPIP_LINK', 'uint32'),
+               ('IFLA_IPIP_LOCAL', 'ip4addr'),
+               ('IFLA_IPIP_REMOTE', 'ip4addr'),
+               ('IFLA_IPIP_TTL', 'uint8'),
+               ('IFLA_IPIP_TOS', 'uint8'),
+               ('IFLA_IPIP_ENCAP_LIMIT', 'uint8'),
+               ('IFLA_IPIP_FLOWINFO', 'be32'),
+               ('IFLA_IPIP_FLAGS', 'be16'),
+               ('IFLA_IPIP_PROTO', 'uint8'),
+               ('IFLA_IPIP_PMTUDISC', 'uint8'),
+               ('IFLA_IPIP_6RD_PREFIX', 'ip6addr'),
+               ('IFLA_IPIP_6RD_RELAY_PREFIX', 'ip4addr'),
+               ('IFLA_IPIP_6RD_PREFIXLEN', 'uint16'),
+               ('IFLA_IPIP_6RD_RELAY_PREFIXLEN', 'uint16'),
+               ('IFLA_IPIP_ENCAP_TYPE', 'uint16'),
+               ('IFLA_IPIP_ENCAP_FLAGS', 'uint16'),
+               ('IFLA_IPIP_ENCAP_SPORT', 'be16'),
+               ('IFLA_IPIP_ENCAP_DPORT', 'be16'),
+               ('IFLA_IPIP_COLLECT_METADATA', 'flag'),
+               ('IFLA_IPIP_FWMARK', 'uint32'))
+
+
 class ifinfbase(object):
     '''
     Network interface message.
@@ -351,7 +411,7 @@ class ifinfbase(object):
                ('IFLA_NET_NS_PID', 'uint32'),
                ('IFLA_IFALIAS', 'asciiz'),
                ('IFLA_NUM_VF', 'uint32'),
-               ('IFLA_VFINFO_LIST', 'hex'),
+               ('IFLA_VFINFO_LIST', 'vflist'),
                ('IFLA_STATS64', 'ifstats64'),
                ('IFLA_VF_PORTS', 'hex'),
                ('IFLA_PORT_SELF', 'hex'),
@@ -376,8 +436,8 @@ class ifinfbase(object):
                ('IFLA_EVENT', 'hex'),
                ('IFLA_NEW_NETNSID', 'hex'),
                ('IFLA_IF_NETNSID', 'hex'),
-               ('IFLA_CARRIER_UP_COUNT', 'hex'),
-               ('IFLA_CARRIER_DOWN_COUNT', 'hex'),
+               ('IFLA_CARRIER_UP_COUNT', 'uint32'),
+               ('IFLA_CARRIER_DOWN_COUNT', 'uint32'),
                ('IFLA_NEW_IFINDEX', 'hex'))
 
     @staticmethod
@@ -438,6 +498,92 @@ class ifinfbase(object):
             if self.netns_fd is not None:
                 os.close(self.netns_fd)
 
+    class vflist(nla):
+        nla_map = (('IFLA_VF_INFO_UNSPEC', 'none'),
+                   ('IFLA_VF_INFO', 'vfinfo'))
+
+        class vfinfo(nla):
+            prefix = 'IFLA_VF_'
+            nla_map = (('IFLA_VF_UNSPEC', 'none'),
+                       ('IFLA_VF_MAC', 'vf_mac'),
+                       ('IFLA_VF_VLAN', 'vf_vlan'),
+                       ('IFLA_VF_TX_RATE', 'vf_tx_rate'),
+                       ('IFLA_VF_SPOOFCHK', 'vf_spoofchk'),
+                       ('IFLA_VF_LINK_STATE', 'vf_link_state'),
+                       ('IFLA_VF_RATE', 'vf_rate'),
+                       ('IFLA_VF_RSS_QUERY_EN', 'vf_rss_query_en'),
+                       ('IFLA_VF_STATS', 'vf_stats'),
+                       ('IFLA_VF_TRUST', 'vf_trust'),
+                       ('IFLA_VF_IB_NODE_GUID', 'hex'),
+                       ('IFLA_VF_IB_PORT_GUID', 'hex'),
+                       ('IFLA_VF_VLAN_LIST', 'vf_vlist'))
+
+            class vf_mac(nla):
+                fields = (('vf', 'I'),
+                          ('mac', '32B'))
+
+                def decode(self):
+                    nla.decode(self)
+                    self['mac'] = ':'.join(['%02x' % x for x
+                                            in self['mac'][:6]])
+
+                def encode(self):
+                    self['mac'] = ([int(x, 16) for x
+                                    in self['mac'].split(':')] +
+                                   [0] * 26)
+                    nla.encode(self)
+
+            class vf_vlan(nla):
+                fields = (('vf', 'I'),
+                          ('vlan', 'I'),
+                          ('qos', 'I'))
+
+            class vf_tx_rate(nla):
+                fields = (('vf', 'I'),
+                          ('tx_rate', 'I'))
+
+            class vf_spoofchk(nla):
+                fields = (('vf', 'I'),
+                          ('spoofchk', 'I'))
+
+            class vf_link_state(nla):
+                fields = (('vf', 'I'),
+                          ('link_state', 'I'))
+
+            class vf_rate(nla):
+                fields = (('vf', 'I'),
+                          ('min_tx_rate', 'I'),
+                          ('max_tx_rate', 'I'))
+
+            class vf_rss_query_en(nla):
+                fields = (('vf', 'I'),
+                          ('rss_query_en', 'I'))
+
+            class vf_stats(nla):
+                nla_map = (('IFLA_VF_STATS_RX_PACKETS', 'uint64'),
+                           ('IFLA_VF_STATS_TX_PACKETS', 'uint64'),
+                           ('IFLA_VF_STATS_RX_BYTES', 'uint64'),
+                           ('IFLA_VF_STATS_TX_BYTES', 'uint64'),
+                           ('IFLA_VF_STATS_BROADCAST', 'uint64'),
+                           ('IFLA_VF_STATS_MULTICAST', 'uint64'),
+                           ('IFLA_VF_STATS_PAD', 'uint64'),
+                           ('IFLA_VF_STATS_RX_DROPPED', 'uint64'),
+                           ('IFLA_VF_STATS_TX_DROPPED', 'uint64'))
+
+            class vf_trust(nla):
+                fields = (('vf', 'I'),
+                          ('trust', 'I'))
+
+            class vf_vlist(nla):
+                nla_map = (('IFLA_VF_VLAN_INFO_UNSPEC', 'none'),
+                           ('IFLA_VF_VLAN_INFO', 'ivvi'))
+
+                class ivvi(nla):
+                    fields = (('vf', 'I'),
+                              ('vlan', 'I'),
+                              ('qos', 'I'),
+                              ('proto', '>H'))
+
     class wireless(iw_event):
         pass
 
@@ -472,6 +618,7 @@ class ifinfbase(object):
         return proto_map.get(self['family'], self.hex)
 
     class ifinfo(nla):
+        prefix = 'IFLA_INFO_'
         nla_map = (('IFLA_INFO_UNSPEC', 'none'),
                    ('IFLA_INFO_KIND', 'asciiz'),
                    ('IFLA_INFO_DATA', 'info_data'),
@@ -521,11 +668,41 @@ class ifinfbase(object):
             def info_peer(self, *argv, **kwarg):
                 return ifinfveth
 
+        class ipip_data(iptnl_data):
+            pass
+
+        class sit_data(iptnl_data):
+            nla_map = [(x[0].replace('IPIP', 'SIT'), x[1])
+                       for x in iptnl_data.nla_map]
+
+        class ip6tnl_data(nla):
+            nla_map = (('IFLA_IP6TNL_UNSPEC', 'none'),
+                       ('IFLA_IP6TNL_LINK', 'uint32'),
+                       ('IFLA_IP6TNL_LOCAL', 'ip6addr'),
+                       ('IFLA_IP6TNL_REMOTE', 'ip6addr'),
+                       ('IFLA_IP6TNL_TTL', 'uint8'),
+                       ('IFLA_IP6TNL_TOS', 'uint8'),
+                       ('IFLA_IP6TNL_ENCAP_LIMIT', 'uint8'),
+                       ('IFLA_IP6TNL_FLOWINFO', 'be32'),
+                       ('IFLA_IP6TNL_FLAGS', 'be16'),
+                       ('IFLA_IP6TNL_PROTO', 'uint8'),
+                       ('IFLA_IP6TNL_PMTUDISC', 'uint8'),
+                       ('IFLA_IP6TNL_6RD_PREFIX', 'ip6addr'),
+                       ('IFLA_IP6TNL_6RD_RELAY_PREFIX', 'ip4addr'),
+                       ('IFLA_IP6TNL_6RD_PREFIXLEN', 'uint16'),
+                       ('IFLA_IP6TNL_6RD_RELAY_PREFIXLEN', 'uint16'),
+                       ('IFLA_IP6TNL_ENCAP_TYPE', 'uint16'),
+                       ('IFLA_IP6TNL_ENCAP_FLAGS', 'uint16'),
+                       ('IFLA_IP6TNL_ENCAP_SPORT', 'be16'),
+                       ('IFLA_IP6TNL_ENCAP_DPORT', 'be16'),
+                       ('IFLA_IP6TNL_COLLECT_METADATA', 'flag'),
+                       ('IFLA_IP6TNL_FWMARK', 'uint32'))
+
         class gre_data(nla):
             nla_map = (('IFLA_GRE_UNSPEC', 'none'),
                        ('IFLA_GRE_LINK', 'uint32'),
-                       ('IFLA_GRE_IFLAGS', 'uint16'),
-                       ('IFLA_GRE_OFLAGS', 'uint16'),
+                       ('IFLA_GRE_IFLAGS', 'gre_flags'),
+                       ('IFLA_GRE_OFLAGS', 'gre_flags'),
                        ('IFLA_GRE_IKEY', 'be32'),
                        ('IFLA_GRE_OKEY', 'be32'),
                        ('IFLA_GRE_LOCAL', 'ip4addr'),
@@ -544,6 +721,22 @@ class ifinfbase(object):
                        ('IFLA_GRE_IGNORE_DF', 'uint8'),
                        ('IFLA_GRE_FWMARK', 'uint32'))
 
+            class gre_flags(nla):
+                fields = [('value', '>H')]
+
+                def encode(self):
+                    #
+                    # for details see:
+                    url = 'https://github.com/svinota/pyroute2/issues/531'
+
+                    v = self.value
+                    for flag in GRE_VALUES:
+                        v &= ~flag
+                    if v != 0:
+                        log.warning('possibly incorrect GRE flags, '
+                                    'see %s' % url)
+                    nla.encode(self)
+
         class ip6gre_data(nla):
             # Ostensibly the same as ip6gre_data except that local
             # and remote are ipv6 addrs.
@@ -632,6 +825,9 @@ class ifinfbase(object):
         # IFLA_INFO_DATA plugin system prototype
         data_map = {'macvlan': macvlan_data,
                     'macvtap': macvtap_data,
+                    'ipip': ipip_data,
+                    'sit': sit_data,
+                    'ip6tnl': ip6tnl_data,
                     'gre': gre_data,
                     'gretap': gre_data,
                     'ip6gre': ip6gre_data,
@@ -641,6 +837,8 @@ class ifinfbase(object):
         # expand supported interface types
         data_map.update(data_plugins)
 
+    sql_extend = ((ifinfo, 'IFLA_LINKINFO'), )
+
     @staticmethod
     def af_spec(self, *argv, **kwarg):
         specs = {0: self.af_spec_inet,
