@@ -27,7 +27,11 @@ class vxlan(nla):
                ('IFLA_VXLAN_REMCSUM_RX', 'uint8'),
                ('IFLA_VXLAN_GBP', 'flag'),
                ('IFLA_VXLAN_REMCSUM_NOPARTIAL', 'flag'),
-               ('IFLA_VXLAN_COLLECT_METADATA', 'uint8'))
+               ('IFLA_VXLAN_COLLECT_METADATA', 'uint8'),
+               ('IFLA_VXLAN_LABEL', 'uint32'),
+               ('IFLA_VXLAN_GPE', 'flag'),
+               ('IFLA_VXLAN_TTL_INHERIT', 'flag'),
+               ('IFLA_VXLAN_DF', 'uint8'))
 
     class port_range(nla):
         fields = (('low', '>H'),
