@@ -17,7 +17,8 @@ from pyroute2.netlink.rtnl.fibmsg import FR_ACT_NAMES
 encap_types = {'mpls': 1,
                AF_MPLS: 1,
                'seg6': 5,
-               'bpf': 6}
+               'bpf': 6,
+               'seg6local': 7}
 
 
 class IPRequest(dict):
@@ -195,6 +196,134 @@ class IPRouteRequest(IPRequest):
                         attrs['LWT_BPF_XMIT_HEADROOM'] = value['headroom']
 
             return {'attrs': attrs.items()}
+        '''
+        Seg6 encap header transform. Format samples:
+
+            {'type': 'seg6local',
+             'action': 'End.DT6',
+             'table': '10'}
+
+            {'type': 'seg6local',
+             'action': 'End.B6',
+             'table': '10'
+             'srh': {'segs': '2000::5,2000::6'}}
+        '''
+        if header['type'] == 'seg6local':
+            # Init step
+            ret = {}
+            table = None
+            nh4 = None
+            nh6 = None
+            iif = None  # Actually not used
+            oif = None
+            srh = {}
+            segs = []
+            hmac = None
+            # Parse segs
+            if srh:
+                segs = header['srh']['segs']
+                # If they are in the form in_addr6,in_addr6
+                if isinstance(segs, basestring):
+                    # Create an array with the splitted values
+                    temp = segs.split(',')
+                    # Init segs
+                    segs = []
+                    # Iterate over the values
+                    for seg in temp:
+                        # Discard empty string
+                        if seg != '':
+                            # Add seg to segs
+                            segs.append(seg)
+                # hmac is optional and contains the hmac key
+                hmac = header.get('hmac', None)
+            # Retrieve action
+            action = header['action']
+            if action == 'End.X':
+                # Retrieve nh6
+                nh6 = header['nh6']
+            elif action == 'End.T':
+                # Retrieve table and convert to u32
+                table = header['table'] & 0xffffffff
+            elif action == 'End.DX2':
+                # Retrieve oif and convert to u32
+                oif = header['oif'] & 0xffffffff
+            elif action == 'End.DX6':
+                # Retrieve nh6
+                nh6 = header['nh6']
+            elif action == 'End.DX4':
+                # Retrieve nh6
+                nh4 = header['nh4']
+            elif action == 'End.DT6':
+                # Retrieve table
+                table = header['table']
+            elif action == 'End.B6':
+                # Parse segs
+                segs = header['srh']['segs']
+                # If they are in the form in_addr6,in_addr6
+                if isinstance(segs, basestring):
+                    # Create an array with the splitted values
+                    temp = segs.split(',')
+                    # Init segs
+                    segs = []
+                    # Iterate over the values
+                    for seg in temp:
+                        # Discard empty string
+                        if seg != '':
+                            # Add seg to segs
+                            segs.append(seg)
+                # hmac is optional and contains the hmac key
+                hmac = header.get('hmac', None)
+                srh['segs'] = segs
+                # If hmac is present convert to u32
+                if hmac:
+                    # Add to ret the hmac key
+                    srh['hmac'] = hmac & 0xffffffff
+                srh['mode'] = 'inline'
+            elif action == 'End.B6.Encaps':
+                # Parse segs
+                segs = header['srh']['segs']
+                # If they are in the form in_addr6,in_addr6
+                if isinstance(segs, basestring):
+                    # Create an array with the splitted values
+                    temp = segs.split(',')
+                    # Init segs
+                    segs = []
+                    # Iterate over the values
+                    for seg in temp:
+                        # Discard empty string
+                        if seg != '':
+                            # Add seg to segs
+                            segs.append(seg)
+                # hmac is optional and contains the hmac key
+                hmac = header.get('hmac', None)
+                srh['segs'] = segs
+                if hmac:
+                    # Add to ret the hmac key
+                    srh['hmac'] = hmac & 0xffffffff
+                srh['mode'] = 'encap'
+            # Construct the new object
+            ret = []
+            ret.append(['SEG6_LOCAL_ACTION', {'value': action}])
+            if table:
+                # Add the table to ret
+                ret.append(['SEG6_LOCAL_TABLE', {'value': table}])
+            if nh4:
+                # Add the nh4 to ret
+                ret.append(['SEG6_LOCAL_NH4', {'value': nh4}])
+            if nh6:
+                # Add the nh6 to ret
+                ret.append(['SEG6_LOCAL_NH6', {'value': nh6}])
+            if iif:
+                # Add the iif to ret
+                ret.append(['SEG6_LOCAL_IIF', {'value': iif}])
+            if oif:
+                # Add the oif to ret
+                ret.append(['SEG6_LOCAL_OIF', {'value': oif}])
+            if srh:
+                # Add the srh to ret
+                ret.append(['SEG6_LOCAL_SRH', srh])
+            # Done return the object
+            return {'attrs': ret}
 
     def mpls_rta(self, value):
         ret = []
@@ -240,12 +369,13 @@ class IPRouteRequest(IPRequest):
                 mask = None
                 if len(value) == 1:
                     dst = value[0]
-                    if self.get('family', 0) == AF_INET:
-                        mask = 32
-                    elif self.get('family', 0) == AF_INET6:
-                        mask = 128
-                    else:
-                        self._mask.append('%s_len' % key)
+                    if '%s_len' % key not in self:
+                        if self.get('family', 0) == AF_INET:
+                            mask = 32
+                        elif self.get('family', 0) == AF_INET6:
+                            mask = 128
+                        else:
+                            self._mask.append('%s_len' % key)
                 elif len(value) == 2:
                     dst = value[0]
                     mask = int(value[1])
@@ -308,6 +438,31 @@ class IPRouteRequest(IPRequest):
                                                      value['type']))
                     dict.__setitem__(self, 'encap',
                                      self.encap_header(value))
+                # human-friendly form:
+                #
+                # 'encap': {'type': 'seg6local',
+                #           'action': 'End'}
+                #
+                # 'encap': {'type': 'seg6local',
+                #           'action': 'End.DT6',
+                #           'table': '10'}
+                #
+                # 'encap': {'type': 'seg6local',
+                #           'action': 'End.DX6',
+                #           'nh6': '2000::5'}
+                #
+                # 'encap': {'type': 'seg6local',
+                #           'action': 'End.B6'
+                #           'srh': {'segs': '2000::5,2000::6',
+                #                   'hmac': 0xf}}
+                #
+                # 'type' and 'action' are mandatory
+                elif 'type' in value and 'action' in value:
+                    dict.__setitem__(self, 'encap_type',
+                                     encap_types.get(value['type'],
+                                                     value['type']))
+                    dict.__setitem__(self, 'encap',
+                                     self.encap_header(value))
                 # assume it is a ready-to-use NLA
                 elif 'attrs' in value:
                     dict.__setitem__(self, 'encap', value)
@@ -317,7 +472,7 @@ class IPRouteRequest(IPRequest):
                     set(value.keys()) == set(('addr', 'family')) and \
                     value['family'] in (AF_INET, AF_INET6) and \
                     isinstance(value['addr'], basestring):
-                        dict.__setitem__(self, 'via', value)
+                dict.__setitem__(self, 'via', value)
         elif key == 'metrics':
             if 'attrs' in value:
                 ret = value
@@ -361,10 +516,11 @@ class IPRouteRequest(IPRequest):
                 dict.__setitem__(self, 'multipath', ret)
         elif key == 'family':
             for d in self._mask:
-                if value == AF_INET:
-                    dict.__setitem__(self, d, 32)
-                elif value == AF_INET6:
-                    dict.__setitem__(self, d, 128)
+                if d not in self:
+                    if value == AF_INET:
+                        dict.__setitem__(self, d, 32)
+                    elif value == AF_INET6:
+                        dict.__setitem__(self, d, 128)
             self._mask = []
             dict.__setitem__(self, key, value)
         else:
@@ -428,7 +584,8 @@ class IPLinkRequest(IPRequest):
     into RTNL link request.
     '''
     blacklist = ['carrier',
-                 'carrier_changes']
+                 'carrier_changes',
+                 'info_slave_kind']
 
     # get common ifinfmsg NLAs
     common = []
@@ -480,6 +637,41 @@ class IPLinkRequest(IPRequest):
 
         self.deferred = []
 
+    def set_vf(self, spec):
+        vflist = []
+        if not isinstance(spec, (list, tuple)):
+            spec = (spec, )
+        for vf in spec:
+            vfcfg = []
+            # pop VF index
+            vfid = vf.pop('vf')          # mandatory
+            # pop VLAN spec
+            vlan = vf.pop('vlan', None)  # optional
+            if isinstance(vlan, int):
+                vfcfg.append(('IFLA_VF_VLAN', {'vf': vfid,
+                                               'vlan': vlan}))
+            elif isinstance(vlan, dict):
+                vlan['vf'] = vfid
+                vfcfg.append(('IFLA_VF_VLAN', vlan))
+            elif isinstance(vlan, (list, tuple)):
+                vlist = []
+                for vspec in vlan:
+                    vspec['vf'] = vfid
+                    vlist.append(('IFLA_VF_VLAN_INFO', vspec))
+                vfcfg.append(('IFLA_VF_VLAN_LIST', {'attrs': vlist}))
+            # pop rate spec
+            rate = vf.pop('rate', None)  # optional
+            if rate is not None:
+                rate['vf'] = vfid
+                vfcfg.append(('IFLA_VF_RATE', rate))
+            # create simple VF attrs
+            for attr in vf:
+                vfcfg.append((ifinfmsg.vflist.vfinfo.name2nla(attr),
+                              {'vf': vfid,
+                               attr: vf[attr]}))
+            vflist.append(('IFLA_VF_INFO', {'attrs': vfcfg}))
+        dict.__setitem__(self, 'IFLA_VFINFO_LIST', {'attrs': vflist})
+
     def set_specific(self, key, value):
         # FIXME: vlan hack
         if self.kind == 'vlan' and key == 'vlan_flags':
@@ -545,9 +737,11 @@ class IPLinkRequest(IPRequest):
         except NameError:
             pass
 
-        if key == 'kind' and not self.kind:
+        if key in ('kind', 'info_kind') and not self.kind:
             self.kind = value
             self.flush_deferred()
+        elif key == 'vf':  # SR-IOV virtual function setup
+            self.set_vf(value)
         elif self.kind is None:
             if key in self.common:
                 dict.__setitem__(self, key, value)
