@@ -82,6 +82,7 @@ import os
 import os.path
 import errno
 import ctypes
+import ctypes.util
 import pickle
 import struct
 import traceback
@@ -112,6 +113,8 @@ MS_REC = 16384
 MS_SHARED = 1 << 20
 NETNS_RUN_DIR = '/var/run/netns'
 
+__saved_ns = []
+
 
 def _get_netnspath(name):
     netnspath = name
@@ -140,7 +143,7 @@ def listnetns(nspath=None):
 
 
 def _create(netns, libc=None):
-    libc = libc or ctypes.CDLL('libc.so.6', use_errno=True)
+    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
     netnspath = _get_netnspath(netns)
     netnsdir = os.path.dirname(netnspath)
 
@@ -205,7 +208,7 @@ def remove(netns, libc=None):
     '''
     Remove a network namespace.
     '''
-    libc = libc or ctypes.CDLL('libc.so.6', use_errno=True)
+    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
     netnspath = _get_netnspath(netns)
     libc.umount2(netnspath, MNT_DETACH)
     os.unlink(netnspath)
@@ -221,11 +224,18 @@ def setns(netns, flags=os.O_CREAT, libc=None):
         - O_CREAT -- create netns, if doesn't exist
         - O_CREAT | O_EXCL -- create only if doesn't exist
 
+    Note that "main" netns has no name. But you can access it with::
+
+        setns('foo')  # move to netns foo
+        setns('/proc/1/ns/net')  # go back to default netns
+
+    See also `pushns()`/`popns()`/`dropns()`
+
     Changed in 0.5.1: the routine closes the ns fd if it's
     not provided via arguments.
     '''
     newfd = False
-    libc = libc or ctypes.CDLL('libc.so.6', use_errno=True)
+    libc = libc or ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
     if isinstance(netns, basestring):
         netnspath = _get_netnspath(netns)
         if os.path.basename(netns) in listnetns(os.path.dirname(netns)):
@@ -247,3 +257,46 @@ def setns(netns, flags=os.O_CREAT, libc=None):
         os.close(nsfd)
     if error != 0:
         raise OSError(ctypes.get_errno(), 'failed to open netns', netns)
+
+
+def pushns(newns=None, libc=None):
+    '''
+    Save the current netns in order to return to it later. If newns is
+    specified, change to it::
+
+        # --> the script in the "main" netns
+        netns.pushns("test")
+        # --> changed to "test", the "main" is saved
+        netns.popns()
+        # --> "test" is dropped, back to the "main"
+    '''
+    global __saved_ns
+    __saved_ns.append(os.open('/proc/self/ns/net', os.O_RDONLY))
+    if newns is not None:
+        setns(newns, libc=libc)
+
+
+def popns(libc=None):
+    '''
+    Restore the previously saved netns.
+    '''
+    global __saved_ns
+    fd = __saved_ns.pop()
+    try:
+        setns(fd, libc=libc)
+    except Exception:
+        __saved_ns.append(fd)
+        raise
+    os.close(fd)
+
+
+def dropns(libc=None):
+    '''
+    Discard the last saved with `pushns()` namespace
+    '''
+    global __saved_ns
+    fd = __saved_ns.pop()
+    try:
+        os.close(fd)
+    except Exception:
+        pass
